@@ -47,13 +47,16 @@
 #define _BSD_SOURCE 1
 #endif
 
-#if defined(BSD) || defined (__OpenBSD__) || defined(__FreeBSD__)
+#if defined (__OpenBSD__)
 #define timeval bpf_timeval
+#endif
+
+#if defined(BSD) || defined (__OpenBSD__) || defined(__FreeBSD__)
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <net/if.h>
 #include <netinet/in.h>
 #include <netinet/in_systm.h>
-#include <net/if.h>
 #endif
 
 #include <arpa/inet.h>
@@ -61,6 +64,7 @@
 #include <netinet/ip.h>
 #include <netinet/tcp.h>
 #include <netinet/udp.h>
+#include "vector.h"
 
 #ifdef INET6_ADDRSTRLEN
 #define ADDRESSLEN INET6_ADDRSTRLEN + 1
@@ -68,7 +72,6 @@
 #define ADDRESSLEN 47
 #endif
 
-
 //! Capture modes
 enum capture_status {
     CAPTURE_ONLINE = 0,
@@ -81,6 +84,19 @@ enum capture_status {
 typedef struct capture_info capture_info_t;
 //! Shorter declaration of dns_cache structure
 typedef struct dns_cache dns_cache_t;
+//! Shorter declaration of capture_packet structure
+typedef struct capture_packet capture_packet_t;
+
+//! Stored packet types
+enum capture_packet_type
+{
+    CAPTURE_PACKET_SIP_UDP = 0,
+    CAPTURE_PACKET_SIP_TCP,
+    CAPTURE_PACKET_SIP_TLS,
+    CAPTURE_PACKET_SIP_WS,
+    CAPTURE_PACKET_SIP_WSS,
+    CAPTURE_PACKET_RTP,
+};
 
 /**
  * @brief Storage for DNS resolved ips
@@ -105,6 +121,8 @@ struct capture_info {
     int status;
     //! Calls capture limit. 0 for disabling
     int limit;
+    //! Also capture RTP packets
+    int rtp_capture;
     //! Input file in Offline capture
     const char *infile;
     //! Key file for TLS decrypt
@@ -129,6 +147,21 @@ struct capture_info {
     pthread_t capture_t;
 };
 
+struct capture_packet {
+    // Packet type as defined in capture_packet_type
+    int type;
+    //! PCAP Packet Header data
+    struct pcap_pkthdr *header;
+    //! PCAP Packet content
+    u_char *data;
+    //! PPCAP Packet content len
+    int size;
+    //! Byte number where payload starts
+    int payload_start;
+    //! Payload length
+    int payload_len;
+};
+
 /**
  * @brief Online capture function
  *
@@ -200,13 +233,13 @@ int
 capture_set_bpf_filter(const char *filter);
 
 /**
- * @brief Limit the number of calls will be captured
+ * @brief Set capture options
  *
  * @param limit Numbers of calls >0
- * @return void
+ * @param rtp_catpure Enable rtp capture
  */
 void
-capture_set_limit(int limit);
+capture_set_opts(int limit, int rtp_capture);
 
 /**
  * @brief Pause/Resume capture
@@ -267,6 +300,36 @@ capture_set_keyfile(const char *keyfile);
 char *
 capture_last_error();
 
+/**
+ * @brief Allocate memory to store new packet data
+ */
+capture_packet_t *
+capture_packet_create(const struct pcap_pkthdr *header, const u_char *packet, int size, int payload_len);
+
+/**
+ * @brief Deallocate a packet structure memory
+ */
+void
+capture_packet_destroy(capture_packet_t *packet);
+
+/**
+ * @brief Destroyer function for packet vectors
+ */
+void
+capture_packet_destroyer(void *packet);
+
+/**
+ * @brief Set packet type
+ */
+void
+capture_packet_set_type(capture_packet_t *packet, int type);
+
+/**
+ * @brief Sorter by time for captured packets
+ */
+void
+capture_packet_time_sorter(vector_t *vector, void *item);
+
 /**
  * @brief Close pcap handler
  */
