@@ -91,32 +91,17 @@ filter_get(int type)
     return filters[type].expr;
 }
 
-void
-filter_stats(int *total, int *displayed)
-{
-    sip_call_t *call = NULL;
-
-    // Initialize stats
-    *total = 0;
-    *displayed = 0;
-
-    while ((call = call_get_next(call))) {
-        (*total)++;
-        if (filter_check_call(call) == 0)
-            (*displayed)++;
-    }
-}
-
 int
-filter_check_call(sip_call_t *call)
+filter_check_call(void *item)
 {
     int i;
     const char *data;
     char linetext[256];
+    sip_call_t *call = (sip_call_t*) item;
 
     // Filter for this call has already be processed
     if (call->filtered != -1)
-        return call->filtered;
+        return (call->filtered == 0);
 
     // By default, call matches all filters
     call->filtered = 0;
@@ -171,15 +156,16 @@ filter_check_call(sip_call_t *call)
     }
 
     // Return the final filter status
-    return call->filtered;
+    return (call->filtered == 0);
 }
 
 void
 filter_reset_calls()
 {
-    sip_call_t *call = NULL;
+    sip_call_t *call;
+    vector_iter_t calls = sip_calls_iterator();
 
     // Force filter evaluation
-    while ((call = call_get_next(call)))
+    while ((call = vector_iterator_next(&calls)))
         call->filtered = -1;
 }
