@@ -45,6 +45,7 @@ call_group_create()
 void
 call_group_destroy(sip_call_group_t *group)
 {
+    vector_destroy(group->calls);
     free(group);
 }
 
@@ -142,7 +143,7 @@ call_group_msg_number(sip_call_group_t *group, sip_msg_t *msg)
     int number = 0;
     sip_msg_t *cur = NULL;
     while ((cur = call_group_get_next_msg(group, cur))) {
-        if (group->sdp_only && !msg->sdp)
+        if (group->sdp_only && !msg_has_sdp(msg))
             continue;
 
         if (cur == msg)
@@ -161,17 +162,6 @@ call_group_get_next_msg(sip_call_group_t *group, sip_msg_t *msg)
     sip_call_t *call;
     int i;
 
-
-    // FIXME Performance hack for huge dialogs
-    if (vector_count(group->calls) == 1) {
-        call = vector_first(group->calls);
-        msgs = vector_iterator(call->msgs);
-        vector_iterator_set_current(&msgs, vector_index(call->msgs, msg));
-        if (group->sdp_only)
-            vector_iterator_set_filter(&msgs, msg_has_sdp);
-        return vector_iterator_next(&msgs);
-    }
-
     for (i = 0; i < vector_count(group->calls); i++) {
 
         call = vector_item(group->calls, i);
@@ -188,7 +178,7 @@ call_group_get_next_msg(sip_call_group_t *group, sip_msg_t *msg)
         }
     }
 
-    return next;
+    return sip_parse_msg(next);
 }
 
 sip_msg_t *
@@ -236,10 +226,12 @@ call_group_get_next_stream(sip_call_group_t *group, rtp_stream_t *stream)
 int
 timeval_is_older(struct timeval t1, struct timeval t2)
 {
-    long diff;
-    diff = t2.tv_sec * 1000000 + t2.tv_usec;
-    diff -= t1.tv_sec * 1000000 + t1.tv_usec;
-    return (diff < 0);
+    long long int t1sec, t2sec;
+    t1sec = t1.tv_sec;
+    t1sec = t1sec * 1000000;
+    t2sec = t2.tv_sec;
+    t2sec = t2sec * 1000000;
+    return ((t2sec + t2.tv_usec) - (t1sec + t1.tv_usec) < 0);
 }
 
 int
@@ -250,7 +242,7 @@ sip_msg_is_older(sip_msg_t *one, sip_msg_t *two)
         return 1;
 
     // Otherwise
-    return timeval_is_older(one->pcap_header->ts, two->pcap_header->ts);
+    return timeval_is_older(msg_get_time(one), msg_get_time(two));
 }
 
 int
