@@ -32,7 +32,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include "ui_manager.h"
-#include "option.h"
+#include "setting.h"
 #include "keybinding.h"
 
 //! sngrep keybindings
@@ -50,7 +50,9 @@ key_bindings_init()
     key_bind_action(ACTION_DOWN, KEY_DOWN);
     key_bind_action(ACTION_DOWN, 'k');
     key_bind_action(ACTION_LEFT, KEY_LEFT);
+    key_bind_action(ACTION_LEFT, 'h');
     key_bind_action(ACTION_RIGHT, KEY_RIGHT);
+    key_bind_action(ACTION_RIGHT, 'l');
     key_bind_action(ACTION_DELETE, KEY_DC);
     key_bind_action(ACTION_BACKSPACE, KEY_BACKSPACE);
     key_bind_action(ACTION_BACKSPACE, KEY_BACKSPACE2);
@@ -70,12 +72,14 @@ key_bindings_init()
     key_bind_action(ACTION_NEXT_FIELD, KEY_DOWN);
     key_bind_action(ACTION_RESIZE_SCREEN, KEY_RESIZE);
     key_bind_action(ACTION_CLEAR, KEY_CTRL('U'));
+    key_bind_action(ACTION_CLEAR, KEY_CTRL('W'));
     key_bind_action(ACTION_CLEAR_CALLS, KEY_F(5));
     key_bind_action(ACTION_TOGGLE_SYNTAX, KEY_F(8));
     key_bind_action(ACTION_TOGGLE_SYNTAX, 'C');
     key_bind_action(ACTION_CYCLE_COLOR, KEY_F(7));
     key_bind_action(ACTION_CYCLE_COLOR, 'c');
     key_bind_action(ACTION_SHOW_HOSTNAMES, KEY_F(9));
+    key_bind_action(ACTION_SHOW_ALIAS, 'a');
     key_bind_action(ACTION_TOGGLE_PAUSE, 'p');
     key_bind_action(ACTION_PREV_SCREEN, KEY_ESC);
     key_bind_action(ACTION_PREV_SCREEN, 'q');
@@ -97,6 +101,9 @@ key_bindings_init()
     key_bind_action(ACTION_SHOW_COLUMNS, KEY_F(10));
     key_bind_action(ACTION_SHOW_COLUMNS, 't');
     key_bind_action(ACTION_SHOW_COLUMNS, 'T');
+    key_bind_action(ACTION_SHOW_SETTINGS, KEY_F(8));
+    key_bind_action(ACTION_SHOW_SETTINGS, 'o');
+    key_bind_action(ACTION_SHOW_SETTINGS, 'O');
     key_bind_action(ACTION_COLUMN_MOVE_UP, '-');
     key_bind_action(ACTION_COLUMN_MOVE_DOWN, '+');
     key_bind_action(ACTION_DISP_FILTER, KEY_F(3));
@@ -107,11 +114,11 @@ key_bindings_init()
     key_bind_action(ACTION_SAVE, KEY_F(2));
     key_bind_action(ACTION_SAVE, 's');
     key_bind_action(ACTION_SAVE, 'S');
-    key_bind_action(ACTION_SELECT, ' ');
+    key_bind_action(ACTION_SELECT, KEY_SPACE);
     key_bind_action(ACTION_CONFIRM, KEY_INTRO);
-    key_bind_action(ACTION_TOGGLE_RTP, 'f');
-    key_bind_action(ACTION_TOGGLE_RAW, KEY_F(3));
     key_bind_action(ACTION_TOGGLE_RAW, 't');
+    key_bind_action(ACTION_TOGGLE_MEDIA, KEY_F(3));
+    key_bind_action(ACTION_TOGGLE_MEDIA, 'm');
     key_bind_action(ACTION_INCREASE_RAW, '9');
     key_bind_action(ACTION_DECREASE_RAW, '0');
     key_bind_action(ACTION_RESET_RAW, 'T');
@@ -197,6 +204,7 @@ key_action_id(const char *action)
     if (!strcmp(action, "togglesyntax")) return ACTION_TOGGLE_SYNTAX;
     if (!strcmp(action, "colormode")) return ACTION_CYCLE_COLOR;
     if (!strcmp(action, "togglehostname")) return ACTION_SHOW_HOSTNAMES;
+    if (!strcmp(action, "togglealias")) return ACTION_SHOW_ALIAS;
     if (!strcmp(action, "pause")) return ACTION_TOGGLE_PAUSE;
     if (!strcmp(action, "prevscreen")) return ACTION_PREV_SCREEN;
     if (!strcmp(action, "help")) return ACTION_SHOW_HELP;
@@ -211,7 +219,6 @@ key_action_id(const char *action)
     if (!strcmp(action, "save")) return ACTION_SAVE;
     if (!strcmp(action, "select")) return ACTION_SELECT;
     if (!strcmp(action, "confirm")) return ACTION_CONFIRM;
-    if (!strcmp(action, "rtp")) return ACTION_TOGGLE_RTP;
     if (!strcmp(action, "rawpreview")) return ACTION_TOGGLE_RAW;
     if (!strcmp(action, "morerawpreview")) return ACTION_INCREASE_RAW;
     if (!strcmp(action, "lessrawpreview")) return ACTION_DECREASE_RAW;
@@ -226,16 +233,12 @@ key_action_id(const char *action)
 int
 key_is_printable(int key)
 {
-    return (key > 33 && key < 126) || (key > 160 && key < 255);
+    return key == ' ' || (key > 33 && key < 126) || (key > 160 && key < 255);
 }
 
 const char *
 key_to_str(int key)
 {
-    //! Check if we already have a human readable key
-    if (key_is_printable(key))
-        return keyname(key);
-
     //! Check function keys and Special keys
     switch(key) {
         case KEY_F(1): return "F1";
@@ -251,6 +254,9 @@ key_to_str(int key)
         case KEY_ESC: return "Esc";
         case KEY_INTRO: return "Enter";
         case ' ': return "Space";
+        default:
+            if (key_is_printable(key))
+                return keyname(key);
     }
 
     return "";
@@ -290,7 +296,7 @@ key_from_str(const char *key)
 const char *
 key_action_key_str(int action)
 {
-    if (is_option_enabled("hintkeyalt") && bindings[action].bindcnt > 1) {
+    if (setting_enabled(SETTING_ALTKEY_HINT) && bindings[action].bindcnt > 1) {
         // First alt keybinding
         return key_to_str(bindings[action].keys[1]);
     } else {
