@@ -20,7 +20,7 @@
  **
  ****************************************************************************/
 /**
- * @file capture_rtp.h
+ * @file rtp.h
  * @author Ivan Alonso [aka Kaian] <kaian@irontec.com>
  *
  * @brief Functions to manage rtp captured packets
@@ -29,6 +29,8 @@
 #ifndef __SNGREP_RTP_H
 #define __SNGREP_RTP_H
 
+#include "config.h"
+#include "capture.h"
 #include "media.h"
 
 #define RTP_FORMAT_MASK     0x7F
@@ -80,6 +82,9 @@ stream_add_packet(rtp_stream_t *stream, const char *ip_src, u_short sport, const
 int
 stream_get_count(rtp_stream_t *stream);
 
+struct sip_call *
+stream_get_call(rtp_stream_t *stream);
+
 const char *
 rtp_get_codec(int code, const char *format);
 
