@@ -68,6 +68,7 @@ enum setting_id
     SETTING_CAPTURE_INFILE,
     SETTING_CAPTURE_OUTFILE,
     SETTING_CAPTURE_KEYFILE,
+    SETTING_CAPTURE_RTP,
     SETTING_SIP_NOINCOMPLETE,
     SETTING_SIP_CALLS,
     SETTING_SAVEPATH,
@@ -112,9 +113,10 @@ struct setting_option {
     char *value;
     //! Compa separated valid values
     const char **valuelist;
+    //! Setting set by user config
+    int user;
 };
 
-
 setting_t *
 setting_by_id(int id);
 
