@@ -60,7 +60,8 @@ sip_init(int limit, int only_calls, int no_incomplete)
     calls.ignore_incomplete = no_incomplete;
 
     // Create a vector to store calls
-    calls.list = vector_create(500, 50);
+    calls.list = vector_create(200, 50);
+    vector_set_destroyer(calls.list, call_destroyer);
 
     // Create hash table for callid search
     hcreate(calls.limit);
@@ -84,123 +85,37 @@ sip_init(int limit, int only_calls, int no_incomplete)
     regcomp(&calls.reg_cseq, "^CSeq:[ ]*([0-9]+) .+\r$", match_flags);
     regcomp(&calls.reg_from, "^(From|f):[ ]*[^:]*:(([^@]+)@?[^\r>;]+)", match_flags);
     regcomp(&calls.reg_to, "^(To|t):[ ]*[^:]*:(([^@]+)@?[^\r>;]+)", match_flags);
-    regcomp(&calls.reg_sdp, "^Content-Type:[ ]* application/sdp\r$", match_flags);
-    regcomp(&calls.reg_sdp_addr, "^c=[^ ]+ [^ ]+ (.+)\r$", match_flags);
-    regcomp(&calls.reg_sdp_port, "^m=[^ ]+ ([0-9]+)", match_flags);
-
-}
-
-sip_msg_t *
-sip_msg_create(const char *payload)
-{
-    sip_msg_t *msg;
-
-    if (!(msg = malloc(sizeof(sip_msg_t))))
-        return NULL;
-    memset(msg, 0, sizeof(sip_msg_t));
-    msg->payload = strdup(payload);
-    msg->color = 0;
-    // Create a vector to store sdp
-    msg->medias = vector_create(0, 2);
-    return msg;
 }
 
 void
-sip_msg_destroy(sip_msg_t *msg)
+sip_deinit()
 {
-
-    if (!msg)
-        return;
-
-    // Free message attribute list
-    sip_attr_list_destroy(msg->attrs);
-
-    // Free message payload pointer if not parsed
-    if (msg->payload)
-        free(msg->payload);
-
-    // Free packet data
-    if (msg->pcap_header)
-        free(msg->pcap_header);
-    if (msg->pcap_packet)
-        free(msg->pcap_packet);
-
-    // Free all memory
-    free(msg);
+    // Remove all calls
+    sip_calls_clear();
+    // Remove calls vector
+    vector_destroy(calls.list);
+    // Remove Call-id hash table
+    hdestroy();
+    // Deallocate regular expressions
+    regfree(&calls.reg_method);
+    regfree(&calls.reg_callid);
+    regfree(&calls.reg_xcallid);
+    regfree(&calls.reg_response);
+    regfree(&calls.reg_cseq);
+    regfree(&calls.reg_from);
+    regfree(&calls.reg_to);
+    // Remove calls mutex
+    pthread_mutex_destroy(&calls.lock);
 }
 
-sip_call_t *
-sip_call_create(char *callid)
-{
-    ENTRY entry;
-    int index;
-
-    // Initialize a new call structure
-    sip_call_t *call = malloc(sizeof(sip_call_t));
-    memset(call, 0, sizeof(sip_call_t));
-
-    // Add this call to the call list
-    pthread_mutex_lock(&calls.lock);
-    index = vector_append(calls.list, call);
-    pthread_mutex_unlock(&calls.lock);
-
-    // Store this call in hash table
-    entry.key = strdup(callid);
-    entry.data = (void *) call;
-    hsearch(entry, ENTER);
-
-    // Create a vector to store call messages
-    call->msgs = vector_create(10, 5);
-    // Create a vector to store RTP streams
-    call->streams = vector_create(4, 2);
-
-    // Initialize call filter status
-    call->filtered = -1;
-
-    // Store current call Index
-    call_set_attribute(call, SIP_ATTR_CALLINDEX, "%d", index);
-
-    return call;
-}
-
-void
-sip_call_destroy(sip_call_t *call)
-{
-    sip_msg_t *msg;
-    vector_iter_t msgs;
-
-    // No call to destroy
-    if (!call)
-        return;
-
-    // Remove call from calls list
-    vector_remove(calls.list, call);
-
-    // Remove all call messages
-    msgs = vector_iterator(call->msgs);
-    while ((msg = vector_iterator_next(&msgs))) {
-        sip_msg_destroy(msg);
-        vector_remove(call->msgs, msg);
-    }
-
-
-    // Remove all call attributes
-    sip_attr_list_destroy(call->attrs);
-
-    // Free it!
-    free(call);
-}
 
 char *
-sip_get_callid(const char* payload)
+sip_get_callid(const char* payload, char *callid)
 {
-    char *callid = NULL;
     regmatch_t pmatch[3];
 
     // Try to get Call-ID from payload
     if (regexec(&calls.reg_callid, payload, 3, pmatch, 0) == 0) {
-        // Call-ID is the firts regexp match
-        callid = malloc(pmatch[2].rm_eo - pmatch[2].rm_so + 1);
         // Allocate memory for Call-Id (caller MUST free it)
         memset(callid, 0, pmatch[2].rm_eo - pmatch[2].rm_so + 1);
         // Copy the matching part of payload
@@ -211,123 +126,120 @@ sip_get_callid(const char* payload)
 }
 
 sip_msg_t *
-sip_load_message(const struct pcap_pkthdr *header, const char *src, u_short sport, const char* dst,
-                 u_short dport, u_char *payload)
+sip_load_message(capture_packet_t *packet, const char *src, u_short sport, const char* dst, u_short dport, u_char *payload)
 {
+    ENTRY entry;
     sip_msg_t *msg;
     sip_call_t *call;
-    char *callid;
+    int call_idx;
+    char callid[1024];
+    char msg_src[ADDRESSLEN];
+    char msg_dst[ADDRESSLEN];
 
     // Get the Call-ID of this message
-    if (!(callid = sip_get_callid((const char*) payload))) {
+    if (!sip_get_callid((const char*) payload, callid)) {
         return NULL;
     }
 
     // Create a new message from this data
-    if (!(msg = sip_msg_create((const char*) payload))) {
+    if (!(msg = msg_create((const char*) payload))) {
         return NULL;
     }
 
     // Get Method and request for the following checks
     // There is no need to parse all payload at this point
     // If no response or request code is found, this is not a SIP message
-    if (!msg_get_reqresp(msg)) {
+    if (!sip_get_msg_reqresp(msg, payload)) {
         // Deallocate message memory
-        sip_msg_destroy(msg);
+        msg_destroy(msg);
         return NULL;
     }
 
+    // If payload is encrypted, dup payload
+    if (packet->type == CAPTURE_PACKET_SIP_TLS)
+        msg->payload = (u_char *)strdup((const char *)payload);
+
+    pthread_mutex_lock(&calls.lock);
+    // Find the call for this msg
+    if (!(call = sip_find_by_callid(callid))) {
+
+        // Check if payload matches expression
+        if (!sip_check_match_expression((const char*) payload))
+            goto skip_message;
+
+        // User requested only INVITE starting dialogs
+        if (calls.only_calls && msg->reqresp != SIP_METHOD_INVITE)
+            goto skip_message;
+
+        // Only create a new call if the first msg
+        // is a request message in the following gorup
+        if (calls.ignore_incomplete && msg->reqresp > SIP_METHOD_MESSAGE)
+            goto skip_message;
+
+        // Check if this message is ignored by configuration directive
+        if (sip_check_msg_ignore(msg))
+            goto skip_message;
+
+        // Create the call if not found
+        if (!(call = call_create(callid)))
+            goto skip_message;
+
+        // Store this call in hash table
+        entry.key = strdup(call_get_attribute(call, SIP_ATTR_CALLID));
+        entry.data = (void *) call;
+        hsearch(entry, ENTER);
+
+        // Append this call to the call list
+        pthread_mutex_lock(&calls.lock);
+        call_idx = vector_append(calls.list, call);
+        pthread_mutex_unlock(&calls.lock);
+
+        // Store current call Index
+        call_set_attribute(call, SIP_ATTR_CALLINDEX, "%d", call_idx);
+    }
+
     // Store sorce address. Prefix too long IPv6 addresses with two dots
     if (strlen(src) > 15) {
-        sprintf(msg->src, "..%s", src + strlen(src) - 13);
+        sprintf(msg_src, "..%s", src + strlen(src) - 13);
     } else {
-        strcpy(msg->src, src);
+        strcpy(msg_src, src);
     }
 
     // Store destination address. Prefix too long IPv6 addresses with two dots
     if (strlen(dst) > 15) {
-        sprintf(msg->dst, "..%s", dst + strlen(dst) - 13);
+        sprintf(msg_dst, "..%s", dst + strlen(dst) - 13);
     } else {
-        strcpy(msg->dst, dst);
+        strcpy(msg_dst, dst);
     }
 
-    // Fill message data
-    msg->sport = sport;
-    msg->dport = dport;
+    // Set Source and Destination attributes
+    msg_set_attribute(msg, SIP_ATTR_SRC, "%s:%u", msg_src, sport);
+    msg_set_attribute(msg, SIP_ATTR_DST, "%s:%u", msg_dst, dport);
 
-    // Set message PCAP header
-    msg->pcap_header = malloc(sizeof(struct pcap_pkthdr));
-    memcpy(msg->pcap_header, header, sizeof(struct pcap_pkthdr));
-
-    pthread_mutex_lock(&calls.lock);
-    // Find the call for this msg
-    if (!(call = call_find_by_callid(callid))) {
-
-        // Check if payload matches expression
-        if (!sip_check_match_expression((const char*) payload)) {
-            // Deallocate message memory
-            sip_msg_destroy(msg);
-            pthread_mutex_unlock(&calls.lock);
-            return NULL;
-        }
-
-        // User requested only INVITE starting dialogs
-        if (calls.only_calls) {
-            if (msg->reqresp != SIP_METHOD_INVITE) {
-                // Deallocate message memory
-                sip_msg_destroy(msg);
-                pthread_mutex_unlock(&calls.lock);
-                return NULL;
-            }
-        }
-
-        // Only create a new call if the first msg
-        // is a request message in the following gorup
-        if (calls.ignore_incomplete) {
-            if (msg->reqresp > SIP_METHOD_MESSAGE) {
-                // Deallocate message memory
-                sip_msg_destroy(msg);
-                pthread_mutex_unlock(&calls.lock);
-                return NULL;
-            }
-        }
-
-        // Check if this message is ignored by configuration directive
-        if (sip_check_msg_ignore(msg)) {
-            // Deallocate message memory
-            sip_msg_destroy(msg);
-            pthread_mutex_unlock(&calls.lock);
-            return NULL;
-        }
-
-        // Create the call if not found
-        if (!(call = sip_call_create(callid))) {
-            // Deallocate message memory
-            sip_msg_destroy(msg);
-            pthread_mutex_unlock(&calls.lock);
-            return NULL;
-        }
-    }
-
-    // Set message callid
-    msg_set_attribute(msg, SIP_ATTR_CALLID, callid);
-    // Dellocate callid memory
-    free(callid);
-
-    // Add the message to the found/created call
+    // Add this SIP packet to the message
+    msg_add_packet(msg, packet);
+    // Add the message to the call
     call_add_message(call, msg);
 
-    msg_parse_payload(msg, msg->payload);
-    // Parse media data
-    msg_parse_media(msg);
-
-    // Update Call State
-    call_update_state(call, msg);
-
+    // Always parse first call message
+    if (call_msg_count(call) == 1 || call_is_invite(call)) {
+        // Parse SIP payload
+        sip_parse_msg_payload(msg, payload);
+        // Parse media data
+        sip_parse_msg_media(msg, payload);
+        // Update Call State
+        call_update_state(call, msg);
+    }
     pthread_mutex_unlock(&calls.lock);
-
     // Return the loaded message
     return msg;
+
+skip_message:
+    // Deallocate message memory
+    msg_destroy(msg);
+    pthread_mutex_unlock(&calls.lock);
+    return NULL;
+
 }
 
 int
@@ -356,19 +268,9 @@ sip_calls_stats(int *total, int *displayed)
     pthread_mutex_unlock(&calls.lock);
 }
 
-void
-call_add_message(sip_call_t *call, sip_msg_t *msg)
-{
-    // Set the message owner
-    msg->call = call;
-    // Put this msg at the end of the msg list
-    vector_append(call->msgs, msg);
-    // Store message count
-    call_set_attribute(call, SIP_ATTR_MSGCNT, "%d", vector_count(call->msgs));
-}
 
 sip_call_t *
-call_find_by_callid(const char *callid)
+sip_find_by_callid(const char *callid)
 {
     ENTRY entry, *eptr;
 
@@ -380,7 +282,7 @@ call_find_by_callid(const char *callid)
 }
 
 sip_call_t *
-call_find_by_xcallid(const char *xcallid)
+sip_find_by_xcallid(const char *xcallid)
 {
     const char *cur_xcallid;
     sip_call_t *cur;
@@ -396,17 +298,6 @@ call_find_by_xcallid(const char *xcallid)
     return NULL;
 }
 
-int
-call_msg_count(sip_call_t *call)
-{
-    return vector_count(call->msgs);
-}
-
-int
-msg_media_count(sip_msg_t *msg)
-{
-    return vector_count(msg->medias);
-}
 
 sip_call_t *
 call_get_xcall(sip_call_t *call)
@@ -414,142 +305,19 @@ call_get_xcall(sip_call_t *call)
     sip_call_t *xcall;
     pthread_mutex_lock(&calls.lock);
     if (call_get_attribute(call, SIP_ATTR_XCALLID)) {
-        xcall = call_find_by_callid(call_get_attribute(call, SIP_ATTR_XCALLID));
+        xcall = sip_find_by_callid(call_get_attribute(call, SIP_ATTR_XCALLID));
     } else {
-        xcall = call_find_by_xcallid(call_get_attribute(call, SIP_ATTR_CALLID));
+        xcall = sip_find_by_xcallid(call_get_attribute(call, SIP_ATTR_CALLID));
     }
     pthread_mutex_unlock(&calls.lock);
     return xcall;
 }
 
-sip_msg_t *
-call_get_next_msg(sip_call_t *call, sip_msg_t *msg)
-{
-    sip_msg_t *ret;
-    pthread_mutex_lock(&calls.lock);
-    if (msg == NULL) {
-        ret = vector_first(call->msgs);
-    } else {
-        ret = vector_item(call->msgs, vector_index(call->msgs, msg) + 1);
-    }
-
-    // Parse message if not parsed
-    if (ret && !ret->parsed)
-        msg_parse_payload(ret, ret->payload);
-
-    pthread_mutex_unlock(&calls.lock);
-    return ret;
-}
-
-sip_msg_t *
-call_get_prev_msg(sip_call_t *call, sip_msg_t *msg)
-{
-    sip_msg_t *ret = NULL;
-    pthread_mutex_lock(&calls.lock);
-    ret = vector_item(call->msgs, vector_index(call->msgs, msg) - 1);
-
-    // Parse message if not parsed
-    if (ret && !ret->parsed)
-        msg_parse_payload(ret, ret->payload);
-
-    pthread_mutex_unlock(&calls.lock);
-    return ret;
-}
-
 int
-call_is_active(void *item)
-{
-    // TODO
-    sip_call_t *call = (sip_call_t *)item;
-    return call->active;
-}
-
-int
-msg_has_sdp(void *item)
-{
-    // TODO
-    sip_msg_t *msg = (sip_msg_t *)item;
-    return msg->sdp;
-}
-
-
-void
-call_update_state(sip_call_t *call, sip_msg_t *msg)
-{
-    const char *callstate;
-    char dur[20];
-    int reqresp;
-    sip_msg_t *first;
-
-    // Sanity check
-    if (!call || !call->msgs || !msg)
-        return;
-
-    // Get the first message in the call
-    first = vector_first(call->msgs);
-
-    // Check First message of Call has INVITE method
-    if (first->reqresp != SIP_METHOD_INVITE) {
-        return;
-    }
-
-    // Get current message Method / Response Code
-    reqresp = msg_get_reqresp(msg);
-
-    // If this message is actually a call, get its current state
-    if ((callstate = call_get_attribute(call, SIP_ATTR_CALLSTATE))) {
-        if (!strcmp(callstate, SIP_CALLSTATE_CALLSETUP)) {
-            if (reqresp == 200) {
-                // Alice and Bob are talking
-                call_set_attribute(call, SIP_ATTR_CALLSTATE, SIP_CALLSTATE_INCALL);
-                // Store the timestap where call has started
-                call->active = 1;
-                call->cstart_msg = msg;
-            } else if (reqresp == SIP_METHOD_CANCEL) {
-                // Alice is not in the mood
-                call_set_attribute(call, SIP_ATTR_CALLSTATE, SIP_CALLSTATE_CANCELLED);
-                // Store total call duration
-                call_set_attribute(call, SIP_ATTR_TOTALDUR, timeval_to_duration(first->pcap_header->ts, msg->pcap_header->ts, dur));
-                call->active = 0;
-            } else if (reqresp > 400) {
-                // Bob is not in the mood
-                call_set_attribute(call, SIP_ATTR_CALLSTATE, SIP_CALLSTATE_REJECTED);
-                // Store total call duration
-                call_set_attribute(call, SIP_ATTR_TOTALDUR, timeval_to_duration(first->pcap_header->ts, msg->pcap_header->ts, dur));
-                call->active = 0;
-            }
-        } else if (!strcmp(callstate, SIP_CALLSTATE_INCALL)) {
-            if (reqresp == SIP_METHOD_BYE) {
-                // Thanks for all the fish!
-                call_set_attribute(call, SIP_ATTR_CALLSTATE, SIP_CALLSTATE_COMPLETED);
-                // Store Conversation duration
-                call_set_attribute(call, SIP_ATTR_CONVDUR,
-                                   timeval_to_duration(call->cstart_msg->pcap_header->ts, msg->pcap_header->ts, dur));
-                call->active = 0;
-            }
-        } else if (reqresp == SIP_METHOD_INVITE && strcmp(callstate, SIP_CALLSTATE_INCALL)) {
-            // Call is being setup (after proper authentication)
-            call_set_attribute(call, SIP_ATTR_CALLSTATE, SIP_CALLSTATE_CALLSETUP);
-            call->active = 1;
-        } else {
-            // Store total call duration
-            call_set_attribute(call, SIP_ATTR_TOTALDUR, timeval_to_duration(first->pcap_header->ts, msg->pcap_header->ts, dur));
-        }
-    } else {
-        // This is actually a call
-        if (reqresp == SIP_METHOD_INVITE) {
-            call_set_attribute(call, SIP_ATTR_CALLSTATE, SIP_CALLSTATE_CALLSETUP);
-            call->active = 1;
-        }
-    }
-}
-
-int
-msg_get_reqresp(sip_msg_t *msg)
+sip_get_msg_reqresp(sip_msg_t *msg, const u_char *payload)
 {
     regmatch_t pmatch[3];
     char reqresp[20];
-    const char *payload = msg->payload;
 
     // If not already parsed
     if (!msg->reqresp) {
@@ -557,13 +325,13 @@ msg_get_reqresp(sip_msg_t *msg)
         memset(reqresp, 0, sizeof(reqresp));
 
         // Method & CSeq
-        if (regexec(&calls.reg_method, payload, 2, pmatch, 0) == 0) {
+        if (regexec(&calls.reg_method, (const char *)payload, 2, pmatch, 0) == 0) {
             sprintf(reqresp, "%.*s", pmatch[1].rm_eo - pmatch[1].rm_so, payload + pmatch[1].rm_so);
             msg_set_attribute(msg, SIP_ATTR_METHOD, reqresp);
         }
 
         // Response code
-        if (regexec(&calls.reg_response, payload, 3, pmatch, 0) == 0) {
+        if (regexec(&calls.reg_response, (const char *)payload, 3, pmatch, 0) == 0) {
             msg_set_attribute(msg, SIP_ATTR_METHOD, "%.*s", pmatch[1].rm_eo - pmatch[1].rm_so,
                               payload + pmatch[1].rm_so);
             sprintf(reqresp, "%.*s", pmatch[2].rm_eo - pmatch[2].rm_so, payload + pmatch[2].rm_so);
@@ -576,33 +344,35 @@ msg_get_reqresp(sip_msg_t *msg)
     return msg->reqresp;
 }
 
+sip_msg_t *
+sip_parse_msg(sip_msg_t *msg)
+{
+    if (msg && !msg->cseq) {
+        sip_parse_msg_payload(msg, (u_char*) msg_get_payload(msg));
+    }
+    return msg;
+}
+
 int
-msg_parse_payload(sip_msg_t *msg, const char *payload)
+sip_parse_msg_payload(sip_msg_t *msg, const u_char *payload)
 {
     regmatch_t pmatch[4];
     char date[12], time[20], cseq[11];
 
-    // If message has already been parsed, we've finished
-    if (msg->parsed)
-        return 0;
-
-    // Get Method and request for the following checks
-    msg_get_reqresp(msg);
-
     // CSeq
-    if (regexec(&calls.reg_cseq, payload, 2, pmatch, 0) == 0) {
+    if (regexec(&calls.reg_cseq, (char*)payload, 2, pmatch, 0) == 0) {
         sprintf(cseq, "%.*s", pmatch[1].rm_eo - pmatch[1].rm_so, payload + pmatch[1].rm_so);
         msg->cseq = atoi(cseq);
     }
 
     // X-Call-Id
-    if (regexec(&calls.reg_xcallid, payload, 3, pmatch, 0) == 0) {
+    if (regexec(&calls.reg_xcallid, (const char *)payload, 3, pmatch, 0) == 0) {
         msg_set_attribute(msg, SIP_ATTR_XCALLID, "%.*s", pmatch[2].rm_eo - pmatch[2].rm_so,
                           payload + pmatch[2].rm_so);
     }
 
     // From
-    if (regexec(&calls.reg_from, payload, 4, pmatch, 0) == 0) {
+    if (regexec(&calls.reg_from, (const char *)payload, 4, pmatch, 0) == 0) {
         msg_set_attribute(msg, SIP_ATTR_SIPFROM, "%.*s", pmatch[2].rm_eo - pmatch[2].rm_so,
                           payload + pmatch[2].rm_so);
         msg_set_attribute(msg, SIP_ATTR_SIPFROMUSER, "%.*s", pmatch[3].rm_eo - pmatch[3].rm_so,
@@ -610,31 +380,35 @@ msg_parse_payload(sip_msg_t *msg, const char *payload)
     }
 
     // To
-    if (regexec(&calls.reg_to, payload, 4, pmatch, 0) == 0) {
+    if (regexec(&calls.reg_to, (const char *)payload, 4, pmatch, 0) == 0) {
         msg_set_attribute(msg, SIP_ATTR_SIPTO, "%.*s", pmatch[2].rm_eo - pmatch[2].rm_so,
                           payload + pmatch[2].rm_so);
         msg_set_attribute(msg, SIP_ATTR_SIPTOUSER, "%.*s", pmatch[3].rm_eo - pmatch[3].rm_so,
                           payload + pmatch[3].rm_so);
     }
 
-    // Set Source and Destination attributes
-    msg_set_attribute(msg, SIP_ATTR_SRC, "%s:%u", msg->src, msg->sport);
-    msg_set_attribute(msg, SIP_ATTR_DST, "%s:%u", msg->dst, msg->dport);
-
     // Set message Date and Time attribute
-    msg_set_attribute(msg, SIP_ATTR_DATE, timeval_to_date(msg->pcap_header->ts, date));
-    msg_set_attribute(msg, SIP_ATTR_TIME, timeval_to_time(msg->pcap_header->ts, time));
+    msg_set_attribute(msg, SIP_ATTR_DATE, timeval_to_date(msg_get_time(msg), date));
+    msg_set_attribute(msg, SIP_ATTR_TIME, timeval_to_time(msg_get_time(msg), time));
 
-    // Message payload has been parsed
-    msg->parsed = 1;
+    capture_packet_t *packet = vector_first(msg->packets);
+    // Store Transport attribute
+    if (packet->type == CAPTURE_PACKET_SIP_UDP) {
+        msg_set_attribute(msg, SIP_ATTR_TRANSPORT, "UDP");
+    } else if (packet->type == CAPTURE_PACKET_SIP_TCP) {
+        msg_set_attribute(msg, SIP_ATTR_TRANSPORT, "TCP");
+    } else if (packet->type == CAPTURE_PACKET_SIP_TLS) {
+        msg_set_attribute(msg, SIP_ATTR_TRANSPORT, "TLS");
+    } else if (packet->type == CAPTURE_PACKET_SIP_WS) {
+        msg_set_attribute(msg, SIP_ATTR_TRANSPORT, "WS");
+    }
 
     return 0;
 }
 
 void
-msg_parse_media(sip_msg_t *msg)
+sip_parse_msg_media(sip_msg_t *msg, const u_char *payload)
 {
-    regmatch_t pmatch[4];
     char address[ADDRESSLEN];
     char media_address[ADDRESSLEN] = { };
     char media_type[15] = { };
@@ -643,38 +417,14 @@ msg_parse_media(sip_msg_t *msg)
     int media_fmt_pref;
     int media_fmt_code;
     sdp_media_t *media = NULL;
-    int port = 0;
-    char *payload, *tofree, *line;
-
-    // Check if this message has sdp
-    if (regexec(&calls.reg_sdp, msg->payload, 0, 0, 0) != 0)
-        return;
+    char *payload2, *tofree, *line;
 
     // Initialize variables
     memset(address, 0, sizeof(address));
 
-    // SDP Address
-    if (regexec(&calls.reg_sdp_addr, msg->payload, 2, pmatch, 0) == 0) {
-        sprintf(address, "%.*s", pmatch[1].rm_eo - pmatch[1].rm_so, msg->payload + pmatch[1].rm_so);
-        msg_set_attribute(msg, SIP_ATTR_SDP_ADDRESS, address);
-    }
-
-    // SDP Port
-    if (regexec(&calls.reg_sdp_port, msg->payload, 2, pmatch, 0) == 0) {
-        msg_set_attribute(msg, SIP_ATTR_SDP_PORT, "%.*s", pmatch[1].rm_eo - pmatch[1].rm_so,
-                          msg->payload + pmatch[1].rm_so);
-        port = atoi(msg_get_attribute(msg, SIP_ATTR_SDP_PORT));
-    }
-
-    if (!strlen(address) || !port)
-        return;
-
-    // Message has SDP
-    msg->sdp = 1;
-
     // Parse each line of payload looking for sdp information
-    tofree = payload = strdup(msg->payload);
-    while ((line = strsep(&payload, "\r\n")) != NULL) {
+    tofree = payload2 = strdup((char*)payload);
+    while ((line = strsep(&payload2, "\r\n")) != NULL) {
         // Check if we have a media string
         if (!strncmp(line, "m=", 2)) {
             if (sscanf(line, "m=%s %d RTP/AVP %d", media_type, &media_port, &media_fmt_pref) == 3) {
@@ -684,7 +434,7 @@ msg_parse_media(sip_msg_t *msg)
                     media_set_port(media, media_port);
                     media_set_address(media, media_address);
                     media_set_format_code(media, media_fmt_pref);
-                    vector_append(msg->medias, media);
+                    msg_add_media(msg, media);
 
                     /**
                      * From SDP we can only guess destination address port. RTP Capture process
@@ -692,8 +442,8 @@ msg_parse_media(sip_msg_t *msg)
                      * and port of the stream.
                      */
                     // Create a new stream with this destination address:port
-                    if (!msg_is_retrans(msg)) {
-                        vector_append(msg->call->streams, stream_create(media, media_address, media_port));
+                    if (!call_msg_is_retrans(msg)) {
+                        call_add_stream(msg_get_call(msg), stream_create(media, media_address, media_port));
                     }
                 }
             }
@@ -716,69 +466,25 @@ msg_parse_media(sip_msg_t *msg)
         }
     }
     free(tofree);
-}
 
-int
-msg_is_retrans(sip_msg_t *msg)
-{
-    sip_msg_t *prev = NULL;
-    vector_iter_t it;
-
-    // Sanity check
-    if (!msg || !msg->call || !msg->payload)
-        return 0;
-
-    // Get previous message in call
-    it = vector_iterator(msg->call->msgs);
-    vector_iterator_set_current(&it, vector_index(msg->call->msgs, msg));
-    prev = vector_iterator_prev(&it);
-
-    return (prev && !strcasecmp(msg->payload, prev->payload));
-}
-
-int
-msg_is_request(sip_msg_t *msg)
-{
-    return msg->reqresp < SIP_METHOD_SENTINEL;
-}
-
-char *
-msg_get_header(sip_msg_t *msg, char *out)
-{
-    // Source and Destination address
-    char from_addr[80], to_addr[80];
-
-    // We dont use Message attributes here because it contains truncated data
-    // This should not overload too much as all results should be already cached
-    sprintf(from_addr, "%s:%u", sip_address_format(msg->src), msg->sport);
-    sprintf(to_addr, "%s:%u", sip_address_format(msg->dst), msg->dport);
-
-    // Get msg header
-    sprintf(out, "%s %s %s -> %s", DATE(msg), TIME(msg), from_addr, to_addr);
-    return out;
-}
-
-struct timeval
-msg_get_time(sip_msg_t *msg)
-{
-    if (msg) {
-        return msg->pcap_header->ts;
-    } else {
-        struct timeval t = { };
-        return t;
+    // If message has media
+    if ((media = vector_first(msg->medias))) {
+        msg_set_attribute(msg, SIP_ATTR_SDP_ADDRESS, media_get_address(media));
+        msg_set_attribute(msg, SIP_ATTR_SDP_PORT, "%d", media_get_port(media));
     }
+
 }
 
+
 void
 sip_calls_clear()
 {
     pthread_mutex_lock(&calls.lock);
-    // Clear all elements in vector
+    // Remove all items from vector
     vector_clear(calls.list);
     // Create again the callid hash table
     hdestroy();
     hcreate(calls.limit);
-
     pthread_mutex_unlock(&calls.lock);
 }
 
@@ -833,62 +539,6 @@ sip_check_match_expression(const char *payload)
 #endif
 }
 
-void
-call_set_attribute(sip_call_t *call, enum sip_attr_id id, const char *fmt, ...)
-{
-    char value[512];
-
-    // Get the actual value for the attribute
-    va_list ap;
-    va_start(ap, fmt);
-    vsprintf(value, fmt, ap);
-    va_end(ap);
-
-    sip_attr_set(call->attrs, id, value);
-}
-
-const char *
-call_get_attribute(sip_call_t *call, enum sip_attr_id id)
-{
-    if (!call)
-        return NULL;
-
-    switch (id) {
-        case SIP_ATTR_CALLINDEX:
-        case SIP_ATTR_MSGCNT:
-        case SIP_ATTR_CALLSTATE:
-        case SIP_ATTR_CONVDUR:
-        case SIP_ATTR_TOTALDUR:
-            return sip_attr_get(call->attrs, id);
-        default:
-            return msg_get_attribute(vector_first(call->msgs), id);
-    }
-
-    return NULL;
-}
-
-void
-msg_set_attribute(sip_msg_t *msg, enum sip_attr_id id, const char *fmt, ...)
-{
-    char value[512];
-
-    // Get the actual value for the attribute
-    va_list ap;
-    va_start(ap, fmt);
-    vsprintf(value, fmt, ap);
-    va_end(ap);
-
-    sip_attr_set(msg->attrs, id, value);
-}
-
-const char *
-msg_get_attribute(sip_msg_t *msg, enum sip_attr_id id)
-{
-    if (!msg)
-        return NULL;
-
-    return sip_attr_get(msg->attrs, id);
-}
 
 int
 sip_check_msg_ignore(sip_msg_t *msg)
@@ -954,10 +604,25 @@ sip_method_from_str(const char *method)
     return atoi(method);
 }
 
+char *
+sip_get_msg_header(sip_msg_t *msg, char *out)
+{
+    // Source and Destination address
+    char from_addr[80], to_addr[80];
+
+    // We dont use Message attributes here because it contains truncated data
+    // This should not overload too much as all results should be already cached
+    sprintf(from_addr, "%s", sip_address_port_format(SRC(msg)));
+    sprintf(to_addr, "%s", sip_address_port_format(DST(msg)));
+
+    // Get msg header
+    sprintf(out, "%s %s %s -> %s", DATE(msg), TIME(msg), from_addr, to_addr);
+    return out;
+}
+
 const char *
 sip_address_format(const char *address)
 {
-
     // Return address formatted depending on active settings
     if (setting_enabled(SETTING_DISPLAY_ALIAS)) {
         return get_alias_value(address);
@@ -971,16 +636,14 @@ sip_address_format(const char *address)
 const char *
 sip_address_port_format(const char *addrport)
 {
-    static char aport[50];
-    char address[50];
+    static char aport[ADDRESSLEN + 6];
+    char address[ADDRESSLEN + 6];
     int port;
 
-    strncpy(aport, addrport, 50);
+    strncpy(aport, addrport, sizeof(aport));
     if (sscanf(aport, "%[^:]:%d", address, &port) == 2) {
         sprintf(aport, "%s:%d", sip_address_format(address), port);
     }
 
     return aport;
 }
-
-
