@@ -72,6 +72,7 @@ ui_t ui_call_flow = {
     .type = PANEL_CALL_FLOW,
     .panel = NULL,
     .create = call_flow_create,
+    .destroy = call_flow_destroy,
     .draw = call_flow_draw,
     .handle_key = call_flow_handle_key,
     .help = call_flow_help
@@ -110,15 +111,33 @@ void
 call_flow_destroy(PANEL *panel)
 {
     call_flow_info_t *info;
-    // Hide the panel
-    hide_panel(panel);
+    call_flow_column_t *column;
+    call_flow_arrow_t *arrow;
+
     // Free the panel information
     if ((info = call_flow_info(panel))) {
-        // Deallocate group memory
+        // Delete panel columns;
+        while ((column = info->columns)) {
+            info->columns = column->next;
+            free(column);
+        }
+
+        // Delete panel arrows
+        while ((arrow = info->arrows)) {
+            info->arrows = arrow->next;
+            free(arrow);
+        }
+
+        // Delete panel windows
+        delwin(info->flow_win);
+        delwin(info->raw_win);
         free(info);
+
     }
     // Delete panel window
     delwin(panel_window(panel));
+    // Deallocate panel pointer
+    del_panel(panel);
 }
 
 call_flow_info_t *
@@ -240,8 +259,8 @@ call_flow_draw_columns(PANEL *panel)
     // Load columns
     for (msg = call_group_get_next_msg(info->group, NULL); msg;
          msg = call_group_get_next_msg(info->group, msg)) {
-        call_flow_column_add(panel, CALLID(msg), SRC(msg));
-        call_flow_column_add(panel, CALLID(msg), DST(msg));
+        call_flow_column_add(panel, call_get_attribute(msg_get_call(msg), SIP_ATTR_CALLID), SRC(msg));
+        call_flow_column_add(panel, call_get_attribute(msg_get_call(msg), SIP_ATTR_CALLID), DST(msg));
     }
 
     // Add RTP columns FIXME Really
@@ -295,6 +314,7 @@ call_flow_draw_message(PANEL *panel, call_flow_arrow_t *arrow, int cline)
     char mediastr[40];
     sip_msg_t *msg = arrow->msg;
     vector_iter_t medias;
+    int color = 0;
 
     // Get panel information
     info = call_flow_info(panel);
@@ -314,7 +334,7 @@ call_flow_draw_message(PANEL *panel, call_flow_arrow_t *arrow, int cline)
 
     // Get message attributes
     msg_time = msg_get_attribute(msg, SIP_ATTR_TIME);
-    msg_callid = msg_get_attribute(msg, SIP_ATTR_CALLID);
+    msg_callid = call_get_attribute(msg_get_call(msg), SIP_ATTR_CALLID);
     msg_method = msg_get_attribute(msg, SIP_ATTR_METHOD);
     msg_src = msg_get_attribute(msg, SIP_ATTR_SRC);
     msg_dst = msg_get_attribute(msg, SIP_ATTR_DST);
@@ -346,18 +366,18 @@ call_flow_draw_message(PANEL *panel, call_flow_arrow_t *arrow, int cline)
     sprintf(method, "%s", msg_method);
 
     // If message has sdp information
-    if (msg->sdp && setting_has_value(SETTING_CF_SDP_INFO, "off")) {
+    if (msg_has_sdp(msg) && setting_has_value(SETTING_CF_SDP_INFO, "off")) {
         // Show sdp tag in tittle
         sprintf(method, "%s (SDP)", msg_method);
     }
 
-    if (msg->sdp && setting_has_value(SETTING_CF_SDP_INFO, "first")) {
+    if (msg_has_sdp(msg) && setting_has_value(SETTING_CF_SDP_INFO, "first")) {
         sprintf(method, "%.3s (%s:%s)",
                 msg_method,
                 msg_get_attribute(msg, SIP_ATTR_SDP_ADDRESS),
                 msg_get_attribute(msg, SIP_ATTR_SDP_PORT));
     }
-    if (msg->sdp && setting_has_value(SETTING_CF_SDP_INFO, "full")) {
+    if (msg_has_sdp(msg) && setting_has_value(SETTING_CF_SDP_INFO, "full")) {
         sprintf(method, "%.3s (%s)",
                 msg_method,
                 msg_get_attribute(msg, SIP_ATTR_SDP_ADDRESS));
@@ -400,17 +420,17 @@ call_flow_draw_message(PANEL *panel, call_flow_arrow_t *arrow, int cline)
     // Color the message {
     if (setting_has_value(SETTING_COLORMODE, "request")) {
         // Color by request / response
-        msg->color = (msg_is_request(msg)) ? CP_RED_ON_DEF : CP_GREEN_ON_DEF;
+        color = (msg_is_request(msg)) ? CP_RED_ON_DEF : CP_GREEN_ON_DEF;
     } else if (setting_has_value(SETTING_COLORMODE, "callid")) {
         // Color by call-id
-        msg->color = call_group_color(info->group, msg->call);
+        color = call_group_color(info->group, msg->call);
     } else if (setting_has_value(SETTING_COLORMODE, "cseq")) {
         // Color by CSeq within the same call
-        msg->color = msg->cseq % 7 + 1;
+        color = msg->cseq % 7 + 1;
     }
 
     // Turn on the message color
-    wattron(win, COLOR_PAIR(msg->color));
+    wattron(win, COLOR_PAIR(color));
 
     // Clear the line
     mvwprintw(win, cline, startpos + 2, "%*s", distance, "");
@@ -418,7 +438,7 @@ call_flow_draw_message(PANEL *panel, call_flow_arrow_t *arrow, int cline)
     mvwprintw(win, cline, startpos + distance / 2 - msglen / 2 + 2, "%.26s", method);
     cline ++;
     // Draw media information
-    if (msg->sdp && setting_has_value(SETTING_CF_SDP_INFO, "full")) {
+    if (msg_has_sdp(msg) && setting_has_value(SETTING_CF_SDP_INFO, "full")) {
         medias = vector_iterator(msg->medias);
         while ((media = vector_iterator_next(&medias))) {
             sprintf(mediastr, "%s %d (%s)",
@@ -438,13 +458,13 @@ call_flow_draw_message(PANEL *panel, call_flow_arrow_t *arrow, int cline)
     // Write the arrow at the end of the message (two arros if this is a retrans)
     if (!strcasecmp(msg_src, column1->addr)) {
         mvwaddch(win, cline, endpos - 2, '>');
-        if (msg_is_retrans(msg)) {
+        if (call_msg_is_retrans(msg)) {
             mvwaddch(win, cline, endpos - 3, '>');
             mvwaddch(win, cline, endpos - 4, '>');
         }
     } else {
         mvwaddch(win, cline, startpos + 2, '<');
-        if (msg_is_retrans(msg)) {
+        if (call_msg_is_retrans(msg)) {
             mvwaddch(win, cline, startpos + 3, '<');
             mvwaddch(win, cline, startpos + 4, '<');
         }
@@ -537,6 +557,11 @@ call_flow_draw_stream(PANEL *panel, call_flow_arrow_t *arrow, int cline)
             mvwaddch(win, cline, endpos - 2, '>');
         else
             mvwaddch(win, cline, endpos, '>');
+        if (arrow->rtp_count != stream_get_count(stream)) {
+            arrow->rtp_count = stream_get_count(stream);
+            arrow->rtp_ind_pos = (arrow->rtp_ind_pos + 1) % distance;
+            mvwaddch(win, cline, startpos + arrow->rtp_ind_pos + 2, '>');
+        }
     } else {
         mvwprintw(win, cline, endpos  + 1, "%d", stream->sport);
         mvwprintw(win, cline, startpos - 5, "%d", stream->dport);
@@ -544,6 +569,11 @@ call_flow_draw_stream(PANEL *panel, call_flow_arrow_t *arrow, int cline)
             mvwaddch(win, cline, startpos + 2, '<');
         else
             mvwaddch(win, cline, startpos, '<');
+        if (arrow->rtp_count != stream_get_count(stream)) {
+            arrow->rtp_count = stream_get_count(stream);
+            arrow->rtp_ind_pos = (arrow->rtp_ind_pos + 1) % distance;
+            mvwaddch(win, cline, endpos - arrow->rtp_ind_pos - 2, '<');
+        }
     }
 
     wattroff(win, A_BOLD | A_REVERSE);
@@ -566,14 +596,14 @@ call_flow_next_arrow(PANEL *panel, const call_flow_arrow_t *cur)
     if (!cur) {
         memset(&cur_time, 0, sizeof(struct timeval));
     } else if (cur->type == CF_ARROW_SIP) {
-        cur_time = cur->msg->pcap_header->ts;
+        cur_time = msg_get_time(cur->msg);
     } else if (cur->type == CF_ARROW_RTP) {
         cur_time = cur->stream->time;
     }
 
     // Look for the next message
     while ((msg = call_group_get_next_msg(info->group, msg))) {
-        if (timeval_is_older(msg->pcap_header->ts, cur_time)) {
+        if (timeval_is_older(msg_get_time(msg), cur_time)) {
             break;
         }
     }
@@ -610,7 +640,7 @@ call_flow_next_arrow(PANEL *panel, const call_flow_arrow_t *cur)
         next->msg = msg;
     } else {
         /* Determine what goes next */
-        if (timeval_is_older(msg->pcap_header->ts, stream->time)) {
+        if (timeval_is_older(msg_get_time(msg), stream->time)) {
             if ((next = call_flow_arrow_find(panel, stream)))
                 return next;
             // Create a new arrow to store next info
@@ -656,7 +686,7 @@ int
 call_flow_arrow_height(PANEL *panel, const call_flow_arrow_t *arrow)
 {
     if (arrow->type == CF_ARROW_SIP) {
-        if (!arrow->msg->sdp)
+        if (!msg_has_sdp(arrow->msg))
             return 2;
         if (setting_has_value(SETTING_CF_SDP_INFO, "off"))
             return 2;
@@ -839,7 +869,6 @@ call_flow_handle_key(PANEL *panel, int key)
                 break;
             case ACTION_SHOW_FLOW_EX:
                 werase(panel_window(panel));
-                // KEY_X , Display current call flow
                 group = call_group_create();
                 if (call_group_count(info->group) == 1) {
                     call_group_add(group, call_get_xcall(vector_first(info->group->calls)));
