@@ -43,6 +43,7 @@ ui_t ui_call_raw = {
     .type = PANEL_CALL_RAW,
     .panel = NULL,
     .create = call_raw_create,
+    .destroy = call_raw_destroy,
     .draw = call_raw_draw,
     .handle_key = call_raw_handle_key
 };
@@ -71,6 +72,22 @@ call_raw_create()
     return panel;
 }
 
+void
+call_raw_destroy(PANEL *panel)
+{
+    call_raw_info_t *info;
+
+    if ((info = call_raw_info(panel))) {
+        // Delete panel windows
+        delwin(info->pad);
+        free(info);
+    }
+    // Delete panel window
+    delwin(panel_window(panel));
+    // Delete panel
+    del_panel(panel);
+}
+
 call_raw_info_t *
 call_raw_info(PANEL *panel)
 {
@@ -108,6 +125,8 @@ call_raw_print_msg(PANEL *panel, sip_msg_t *msg)
     int payload_lines, i, column, height, width;
     // Message ngrep style Header
     char header[256];
+    char payload[2048];
+    int color = 0;
 
     // Get panel information
     if (!(info = call_raw_info(panel)))
@@ -119,11 +138,14 @@ call_raw_print_msg(PANEL *panel, sip_msg_t *msg)
     // Get current pad dimensions
     getmaxyx(pad, height, width);
 
+    // Get message payload
+    strcpy(payload, msg_get_payload(msg));
+
     // Check how many lines we well need to draw this message
     payload_lines = 0;
     column = 0;
-    for (i = 0; i < strlen(msg->payload); i++) {
-        if (column == width || msg->payload[i] == '\n') {
+    for (i = 0; i < strlen(payload); i++) {
+        if (column == width || payload[i] == '\n') {
             payload_lines++;
             column = 0;
             continue;
@@ -133,12 +155,13 @@ call_raw_print_msg(PANEL *panel, sip_msg_t *msg)
 
     // Check if we have enough space in our huge pad to store this message
     if (info->padline + payload_lines > height) {
-        // Delete previous pad
-        delwin(info->pad);
         // Create a new pad with more lines!
         pad = newpad(height + 500, COLS);
         // And copy all previous information
         overwrite(info->pad, pad);
+        // Delete previous pad
+        delwin(info->pad);
+        // And store the new pad
         info->pad = pad;
     }
 
@@ -146,24 +169,24 @@ call_raw_print_msg(PANEL *panel, sip_msg_t *msg)
     if (setting_has_value(SETTING_COLORMODE, "request")) {
         // Determine arrow color
         if (msg_is_request(msg)) {
-            msg->color = CP_RED_ON_DEF;
+            color = CP_RED_ON_DEF;
         } else {
-            msg->color = CP_GREEN_ON_DEF;
+            color = CP_GREEN_ON_DEF;
         }
     } else if (info->group && setting_has_value(SETTING_COLORMODE, "callid")) {
         // Color by call-id
-        msg->color = call_group_color(info->group, msg->call);
+        color = call_group_color(info->group, msg->call);
     } else if (setting_has_value(SETTING_COLORMODE, "cseq")) {
         // Color by CSeq within the same call
-        msg->color = msg->cseq % 7 + 1;
+        color = msg->cseq % 7 + 1;
     }
 
     // Turn on the message color
-    wattron(pad, COLOR_PAIR(msg->color));
+    wattron(pad, COLOR_PAIR(color));
 
     // Print msg header
     wattron(pad, A_BOLD);
-    mvwprintw(pad, info->padline++, 0, "%s", msg_get_header(msg, header));
+    mvwprintw(pad, info->padline++, 0, "%s", sip_get_msg_header(msg, header));
     wattroff(pad, A_BOLD);
 
     // Print msg payload
