@@ -29,6 +29,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include <regex.h>
+#include <unistd.h>
 #include "ui_manager.h"
 #include "ui_call_list.h"
 #include "ui_column_select.h"
@@ -90,19 +91,19 @@ column_select_create()
     info->menu_win = derwin(win, 10, width - 2, 7, 0);
 
     // Initialize one field for each attribute
-    for (attr_id = 0; attr_id < SIP_ATTR_SENTINEL; attr_id++) {
+    for (attr_id = 0; attr_id < SIP_ATTR_COUNT; attr_id++) {
         // Create a new field for this column
         info->items[attr_id] = new_item("[ ]", sip_attr_get_description(attr_id));
         set_item_userptr(info->items[attr_id], (void*) sip_attr_get_name(attr_id));
     }
-    info->items[SIP_ATTR_SENTINEL] = NULL;
+    info->items[SIP_ATTR_COUNT] = NULL;
 
     // Create the columns menu and post it
     info->menu = menu = new_menu(info->items);
 
     // Set current enabled fields
-    call_list_info_t *list_info = (call_list_info_t*) panel_userptr(
-                                      ui_get_panel(ui_find_by_type(PANEL_CALL_LIST)));
+    // FIXME Stealing Call list columns :/
+    call_list_info_t *list_info = call_list_info(ui_get_panel(ui_find_by_type(PANEL_CALL_LIST)));
 
     // Enable current enabled fields and move them to the top
     for (column = 0; column < list_info->columncnt; column++) {
@@ -159,7 +160,7 @@ void
 column_select_destroy(PANEL *panel)
 {
     int i, itemcnt;
-    column_select_info_t *info = (column_select_info_t*) panel_userptr(panel);
+    column_select_info_t *info = column_select_info(panel);
 
     // Get item count
     itemcnt = item_count(info->menu);
@@ -173,11 +174,18 @@ column_select_destroy(PANEL *panel)
     }
 }
 
+
+column_select_info_t *
+column_select_info(PANEL *panel)
+{
+    return (column_select_info_t*) panel_userptr(panel);
+}
+
 int
 column_select_handle_key(PANEL *panel, int key)
 {
     // Get panel information
-    column_select_info_t *info = (column_select_info_t*) panel_userptr(panel);
+    column_select_info_t *info = column_select_info(panel);
 
     if (info->form_active) {
         return column_select_handle_key_form(panel, key);
@@ -196,7 +204,7 @@ column_select_handle_key_menu(PANEL *panel, int key)
     int action = -1;
 
     // Get panel information
-    column_select_info_t *info = (column_select_info_t*) panel_userptr(panel);
+    column_select_info_t *info = column_select_info(panel);
 
     menu = info->menu;
     current = current_item(menu);
@@ -263,7 +271,7 @@ column_select_handle_key_form(PANEL *panel, int key)
     int action = -1;
 
     // Get panel information
-    column_select_info_t *info = (column_select_info_t*) panel_userptr(panel);
+    column_select_info_t *info = column_select_info(panel);
 
     // Get current field id
     field_idx = field_index(current_field(info->form));
@@ -342,11 +350,11 @@ column_select_update_columns(PANEL *panel)
     int column, attr_id;
 
     // Get panel information
-    column_select_info_t *info = (column_select_info_t*) panel_userptr(panel);
+    column_select_info_t *info = column_select_info(panel);
 
     // Set enabled fields
     PANEL *list_panel = ui_get_panel(ui_find_by_type(PANEL_CALL_LIST));
-    call_list_info_t *list_info = (call_list_info_t*) panel_userptr(list_panel);
+    call_list_info_t *list_info = call_list_info(list_panel);
 
     // Reset column count
     list_info->columncnt = 0;
@@ -374,7 +382,6 @@ column_select_save_columns(PANEL *panel)
 {
     int column;
     FILE *fi, *fo;
-    regex_t setting;
     char columnopt[128];
     char line[1024];
     char *home = getenv("HOME");
@@ -388,6 +395,9 @@ column_select_save_columns(PANEL *panel)
     sprintf(userconf, "%s/.sngreprc", home);
     sprintf(tmpfile, "%s/.sngreprc.old", home);
 
+    // Remove old config file
+    unlink(tmpfile);
+
     // Move home file to temporal dir
     rename(userconf, tmpfile);
 
@@ -398,22 +408,20 @@ column_select_save_columns(PANEL *panel)
 
     // Read all lines of old sngreprc file
     if ((fi = fopen(tmpfile, "r"))) {
-        // Compile expression matching
-        regcomp(&setting, "^\\s*set\\s+cl.column[0-9]+\\s+\\S+\\s*$", REG_EXTENDED);
+
         // Read all configuration file
         while (fgets(line, 1024, fi) != NULL) {
-            // Dont copy set cl.columns lines
-            if (regexec(&setting, line, 0, 0, 0) == 0)
-                continue;
-            // Put everyting in new .sngreprc file
-            fputs(line, fo);
+            // Ignore lines starting with set (but keep settings)
+            if (strncmp(line, "set ", 4) || strncmp(line, "set cl.column", 13)) {
+                // Put everyting in new .sngreprc file
+                fputs(line, fo);
+            }
         }
         fclose(fi);
-        regfree(&setting);
     }
 
     // Get panel information
-    column_select_info_t *info = (column_select_info_t*) panel_userptr(panel);
+    column_select_info_t *info = column_select_info(panel);
 
     // Add all selected columns
     for (column = 0; column < item_count(info->menu); column++) {
@@ -433,7 +441,7 @@ void
 column_select_move_item(PANEL *panel, ITEM *item, int pos)
 {
     // Get panel information
-    column_select_info_t *info = (column_select_info_t*) panel_userptr(panel);
+    column_select_info_t *info = column_select_info(panel);
 
     // Check we have a valid position
     if (pos == item_count(info->menu) || pos < 0)
@@ -462,7 +470,7 @@ void
 column_select_update_menu(PANEL *panel)
 {
     // Get panel information
-    column_select_info_t *info = (column_select_info_t*) panel_userptr(panel);
+    column_select_info_t *info = column_select_info(panel);
     ITEM *current = current_item(info->menu);
     int top_idx = top_row(info->menu);
 
