@@ -30,9 +30,10 @@
 #include <stdlib.h>
 #include <form.h>
 #include "ui_filter.h"
+#include "ui_call_list.h"
 #include "sip.h"
 #include "filter.h"
-#include "option.h"
+#include "setting.h"
 
 /**
  * Ui Structure definition for Filter panel
@@ -128,7 +129,7 @@ filter_create()
 
     // Get Method filter
     if (!(method = filter_get(FILTER_METHOD)))
-        method = get_option_value("filter.methods");
+        method = setting_get_value(SETTING_FILTER_METHODS);
 
     // Set Default field values
     set_field_buffer(info->fields[FLD_FILTER_SIPFROM], 0, filter_get(FILTER_SIPFROM));
@@ -169,6 +170,12 @@ filter_create()
     return panel;
 }
 
+filter_info_t *
+filter_info(PANEL *panel)
+{
+    return (filter_info_t*) panel_userptr(panel);
+}
+
 void
 filter_destroy(PANEL *panel)
 {
@@ -184,7 +191,7 @@ filter_handle_key(PANEL *panel, int key)
     int action = -1;
 
     // Get panel information
-    filter_info_t *info = (filter_info_t*) panel_userptr(panel);
+    filter_info_t *info = filter_info(panel);
 
     // Get current field id
     field_idx = field_index(current_field(info->form));
@@ -200,16 +207,13 @@ filter_handle_key(PANEL *panel, int key)
         // Check if we handle this action
         switch (action) {
             case ACTION_PRINTABLE:
-                // If this is a normal character on input field, print it
-                 switch (field_idx) {
-                     case FLD_FILTER_SIPFROM:
-                     case FLD_FILTER_SIPTO:
-                     case FLD_FILTER_SRC:
-                     case FLD_FILTER_DST:
-                         form_driver(info->form, key);
-                         break;
-                 }
-                 break;
+                 // If this is a normal character on input field, print it
+                if (field_idx == FLD_FILTER_SIPFROM || field_idx == FLD_FILTER_SIPTO
+                        || field_idx == FLD_FILTER_SRC || field_idx == FLD_FILTER_DST) {
+                    form_driver(info->form, key);
+                    break;
+                }
+                continue;
             case ACTION_NEXT_FIELD:
                 form_driver(info->form, REQ_NEXT_FIELD);
                 form_driver(info->form, REQ_END_LINE);
@@ -258,9 +262,11 @@ filter_handle_key(PANEL *panel, int key)
                             form_driver(info->form, '*');
                         }
                         break;
-                    default:
-                        form_driver(info->form, ' ');
-                        break;
+                    case FLD_FILTER_CANCEL:
+                        return KEY_ESC;
+                    case FLD_FILTER_FILTER:
+                        filter_save_options(panel);
+                        return KEY_ESC;
                 }
                 break;
             case ACTION_CONFIRM:
@@ -307,7 +313,7 @@ filter_save_options(PANEL *panel)
     memset(method_expr, 0, sizeof(method_expr));
 
     // Get panel information
-    filter_info_t *info = (filter_info_t*) panel_userptr(panel);
+    filter_info_t *info = filter_info(panel);
 
     for (field_id = 0; field_id < FLD_FILTER_COUNT; field_id++) {
         // Get current field value.
@@ -317,11 +323,7 @@ filter_save_options(PANEL *panel)
         sscanf(field_buffer(info->fields[field_id], 0), "%[^ ]", field_value);
 
         // Set filter expression
-        if (strlen(field_value)) {
-            expr = field_value;
-        } else {
-            expr = NULL;
-        }
+        expr = strlen(field_value) ? field_value : NULL;
 
         switch (field_id) {
             case FLD_FILTER_SIPFROM:
@@ -363,6 +365,9 @@ filter_save_options(PANEL *panel)
 
     // Force filter evaluation
     filter_reset_calls();
+    // TODO FIXME Refresh call list FIXME
+    call_list_clear(ui_get_panel(ui_find_by_type(PANEL_CALL_LIST)));
+
 }
 
 const char*
