@@ -64,7 +64,7 @@ static ui_t *panel_pool[] =
 };
 
 int
-init_interface()
+ncurses_init()
 {
     int bg, fg;
     const char *term;
@@ -139,13 +139,13 @@ init_interface()
     return 0;
 }
 
-int
-deinit_interface()
+void
+ncurses_deinit()
 {
     // Clear screen before leaving
     refresh();
     // End ncurses mode
-    return endwin();
+    endwin();
 }
 
 ui_t *
@@ -173,17 +173,17 @@ ui_create_panel(enum panel_types type)
 void
 ui_destroy(ui_t *ui)
 {
-    PANEL *panel;
-    // If thre is no ui panel, we're done
-    if (!(panel = ui_get_panel(ui)))
+    // If there is no ui panel, we're done
+    if (!ui || !ui->panel)
         return;
 
+    // Hide this panel before destroying
+    hide_panel(ui->panel);
+
     // If panel has a destructor function use it
     if (ui->destroy)
-        ui->destroy(panel);
+        ui->destroy(ui->panel);
 
-    // Deallocate panel pointer
-    del_panel(ui->panel);
     // Initialize panel pointer
     ui->panel = NULL;
 }
@@ -295,17 +295,13 @@ wait_for_input()
     PANEL *panel;
 
     // While there are still panels
-    while (panel_below(NULL)) {
+    while ((panel = panel_below(NULL))) {
 
-        // Redraw all panels
-        panel = NULL;
-        while ((panel = panel_above(panel))) {
-            // Get panel interface structure
-            ui = ui_find_by_panel(panel);
-            // Redraw this panel
-            if (ui_draw_panel(ui) != 0)
-                return -1;
-        }
+        // Get panel interface structure
+        ui = ui_find_by_panel(panel);
+        // Redraw this panel
+        if (ui_draw_panel(ui) != 0)
+            return -1;
 
         // Update panel stack
         update_panels();
@@ -512,7 +508,7 @@ int
 draw_message_pos(WINDOW *win, sip_msg_t *msg, int starting)
 {
     int height, width, line, column, i;
-    char *cur_line = msg->payload;
+    const char *cur_line, *payload;
     int syntax = setting_enabled(SETTING_SYNTAX);
 
     // Default text format
@@ -523,10 +519,13 @@ draw_message_pos(WINDOW *win, sip_msg_t *msg, int starting)
     // Get window of main panel
     getmaxyx(win, height, width);
 
+    // Get packet payload
+    cur_line = payload = (const char *) msg_get_payload(msg);
+
     // Print msg payload
     line = starting;
     column = 0;
-    for (i = 0; i < strlen(msg->payload); i++) {
+    for (i = 0; i < strlen(payload); i++) {
         // If syntax highlighting is enabled
         if (syntax) {
             // First line highlight
@@ -540,13 +539,13 @@ draw_message_pos(WINDOW *win, sip_msg_t *msg, int starting)
                     attrs = A_BOLD | COLOR_PAIR(CP_RED_ON_DEF);
 
                 // SIP URI syntax
-                if (!strncasecmp(msg->payload + i, "sip:", 4)) {
+                if (!strncasecmp(payload + i, "sip:", 4)) {
                     attrs = A_BOLD | COLOR_PAIR(CP_CYAN_ON_DEF);
                 }
             } else {
 
                 // Header syntax
-                if (strchr(cur_line, ':') && msg->payload + i < strchr(cur_line, ':'))
+                if (strchr(cur_line, ':') && payload + i < strchr(cur_line, ':'))
                     attrs = A_NORMAL | COLOR_PAIR(CP_GREEN_ON_DEF);
 
                 // Call-ID Header syntax
@@ -554,20 +553,20 @@ draw_message_pos(WINDOW *win, sip_msg_t *msg, int starting)
                     attrs = A_BOLD | COLOR_PAIR(CP_MAGENTA_ON_DEF);
 
                 // CSeq Heaedr syntax
-                if (!strncasecmp(cur_line, "CSeq:", 5) && column > 5 && !isdigit(msg->payload[i]))
+                if (!strncasecmp(cur_line, "CSeq:", 5) && column > 5 && !isdigit(payload[i]))
                     attrs = A_NORMAL | COLOR_PAIR(CP_YELLOW_ON_DEF);
 
                 // tag and branch syntax
-                if (i > 0 && msg->payload[i - 1] == ';') {
+                if (i > 0 && payload[i - 1] == ';') {
                     // Highlight branch if requested
                     if (setting_enabled(SETTING_SYNTAX_BRANCH)) {
-                        if (!strncasecmp(msg->payload + i, "branch", 6)) {
+                        if (!strncasecmp(payload + i, "branch", 6)) {
                             attrs = A_BOLD | COLOR_PAIR(CP_CYAN_ON_DEF);
                         }
                     }
                     // Highlight tag if requested
                     if (setting_enabled(SETTING_SYNTAX_TAG)) {
-                        if (!strncasecmp(msg->payload + i, "tag", 3)) {
+                        if (!strncasecmp(payload + i, "tag", 3)) {
                             if (!strncasecmp(cur_line, "From:", 5)) {
                                 attrs = A_BOLD | COLOR_PAIR(CP_DEFAULT);
                             } else {
@@ -583,7 +582,7 @@ draw_message_pos(WINDOW *win, sip_msg_t *msg, int starting)
             }
 
             // Remove previous syntax
-            if (strcspn(msg->payload + i, " \n;<>") == 0) {
+            if (strcspn(payload + i, " \n;<>") == 0) {
                 wattroff(win, attrs);
                 attrs = A_NORMAL | COLOR_PAIR(CP_DEFAULT);
             }
@@ -593,22 +592,22 @@ draw_message_pos(WINDOW *win, sip_msg_t *msg, int starting)
         }
 
         // Dont print this characters
-        if (msg->payload[i] == '\r')
+        if (payload[i] == '\r')
             continue;
 
         // Store where the line begins
-        if (msg->payload[i] == '\n')
-            cur_line = msg->payload + i + 1;
+        if (payload[i] == '\n')
+            cur_line =payload + i + 1;
 
         // Move to the next line if line is filled or a we reach a line break
-        if (column > width || msg->payload[i] == '\n') {
+        if (column > width || payload[i] == '\n') {
             line++;
             column = 0;
             continue;
         }
 
         // Put next character in position
-        mvwaddch(win, line, column++, msg->payload[i]);
+        mvwaddch(win, line, column++, payload[i]);
 
         // Stop if we've reached the bottom of the window
         if (line == height)
@@ -681,3 +680,76 @@ dialog_run(const char *fmt, ...)
     return 1;
 
 }
+
+WINDOW *
+dialog_progress_run(const char *fmt, ...)
+{
+    char textva[2048];
+    va_list ap;
+    int height, width;
+    WINDOW *win;
+    char *word;
+    int col = 2;
+    int line = 2;
+
+    // Get the message from the format string
+    va_start(ap, fmt);
+    vsprintf(textva, fmt, ap);
+    va_end(ap);
+
+    // Determine dialog dimensions
+    height = 6 + (strlen(textva) / 50);
+    width = strlen(textva);
+
+    // Check we don't have a too big or small window
+    if (width > DIALOG_MAX_WIDTH)
+        width = DIALOG_MAX_WIDTH;
+    if (width < DIALOG_MIN_WIDTH)
+        width = DIALOG_MIN_WIDTH;
+
+    // Create the window
+    win = newwin(height, width, (LINES - height) / 2, (COLS - width) / 2);
+    box(win, 0, 0);
+
+    // Write the message into the screen
+    for (word = strtok(textva, " "); word; word = strtok(NULL, " ")) {
+        if (col + strlen(word) > width - 2) {
+            line++;
+            col = 2;
+        }
+        mvwprintw(win, line, col, "%s", word);
+        col += strlen(word) + 1;
+    }
+
+    curs_set(0);
+    wrefresh(win);
+    // Disable input timeout
+    nocbreak();
+    cbreak();
+
+    return win;
+
+}
+
+void
+dialog_progress_set_value(WINDOW *win, int perc)
+{
+    int height, width;
+
+    getmaxyx(win, height, width);
+    mvwhline(win, 4, 4, '-', width - 10);
+    mvwaddch(win, 4, 3, '[');
+    mvwaddch(win, 4, width - 7, ']');
+    mvwprintw(win, 4, width - 5, "%d%%", perc);
+
+    if (perc > 0 && perc <= 100)
+        mvwhline(win, 4, 4, ACS_CKBOARD, (width - 10) * ((float)perc/100));
+
+    wrefresh(win);
+}
+
+void
+dialog_progress_destroy(WINDOW *win)
+{
+    delwin(win);
+}
